@@ -2,7 +2,7 @@
  * bookmark.c - bookmarks
  *
  * Copyright (C) 2003 Oskar Swida
- * modifications (C) 2004-2013 Olivier Sessink
+ * modifications (C) 2004-2023 Olivier Sessink
  * modifications (C) 2011-2012 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
  */
 /*#define DEBUG*/
 /*#define BMARKREF*/
+#define SUPPRESS_DRAWING_CALLS /* reduces the output caused by drawing the margin of the document */
 
 #include <gtk/gtk.h>
 #include <fcntl.h>
@@ -247,6 +248,9 @@ bmark_update_treestore_name(Tbfwin * bfwin)
 	}
 }
 
+/* 
+this function is called when drawing the margin in the editor widget, which is *a lot*
+*/
 static void
 bmark_update_offset_from_textmark(Tbmark * b)
 {
@@ -275,6 +279,9 @@ bmark_update_offset_from_textmark(Tbmark * b)
  *
  * returns NULL if we have to append this as first child to the parent
  *
+ * when adings lots (like 1000) bookmarks from the search dialog, this function
+ * uses a considerable amount of time 
+ *
  */
 static Tbmark *
 bmark_find_bookmark_before_offset(Tbfwin * bfwin, guint offset, GtkTreeIter * parent)
@@ -282,6 +289,7 @@ bmark_find_bookmark_before_offset(Tbfwin * bfwin, guint offset, GtkTreeIter * pa
 	gboolean cont;
 	GtkTreeIter iter;
 	Tbmark *b1 = NULL, *b2;
+	DEBUG_MSG("bmark_find_bookmark_before_offset, started\n");
 	cont =
 		gtk_tree_model_iter_children(GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore), &iter,
 									 parent);
@@ -648,28 +656,31 @@ bmark_activate(Tbfwin * bfwin, Tbmark * b, gboolean select_bmark)
 		gtk_text_buffer_get_iter_at_mark(b->doc->buffer, &it, b->mark);
 		b->offset = gtk_text_iter_get_offset(&it);
 	}
-	DEBUG_MSG("bmark_activate, bmark at %p, uri at %p\n", b, b->uri);
-	DEBUG_MSG("bmark_activate, calling doc_new_from_uri with goto_offset %d\n", b->offset);
+	DEBUG_MSG("bmark_activate, bmark=%p, bmark->uri=%p, bmark->doc=%p\n", b, b->uri, b->doc);
 	if (b->doc) {
+		DEBUG_MSG("bmark_activate, calling bfwin_switch_to_document_by_pointer(%p) and doc_select_line_by_offset(%d)\n", b->doc, b->offset);
 		bfwin_switch_to_document_by_pointer(BFWIN(b->doc->bfwin), b->doc);
 		doc_select_line_by_offset(b->doc, b->offset, TRUE, TRUE);
 	} else {
+		DEBUG_MSG("bmark_activate, calling doc_new_from_uri with goto_offset %d\n", b->offset);
 		doc_new_from_uri(bfwin, b->uri, NULL, FALSE, FALSE, -1, b->offset, -1, TRUE, FALSE);
 	}
 	/* remove selection */
 	if (b->doc) {
+		DEBUG_MSG("move selection and grab focus\n");
 		gtk_text_buffer_get_iter_at_mark(b->doc->buffer, &it, gtk_text_buffer_get_insert(b->doc->buffer));
 		gtk_text_buffer_move_mark_by_name(b->doc->buffer, "selection_bound", &it);
 		gtk_widget_grab_focus(b->doc->view);
 	} else if (bfwin->current_document) {
+		DEBUG_MSG("grab focus\n");
 		gtk_widget_grab_focus(bfwin->current_document->view);
 	}
 	if (select_bmark) {
 		GtkTreeIter fiter;
+		DEBUG_MSG("select the iter for the bmark in the treeview\n");
 		gtk_tree_model_filter_convert_child_iter_to_iter(bfwin->bmarkfilter, &fiter, &b->iter);
 		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(bfwin->bmark), &fiter);
 	}
-
 }
 
 /*static void bmark_goto_selected(Tbfwin *bfwin) {
@@ -1051,6 +1062,7 @@ static void
 bmark_row_activated(GtkTreeView * tree, GtkTreePath * path, GtkTreeViewColumn * column, Tbfwin * bfwin)
 {
 	GtkTreeIter iter;
+	DEBUG_MSG("bmark_row_activated, started\n");
 	gtk_tree_model_get_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter, path);
 	if (gtk_tree_path_get_depth(path) == 2) {
 		bmark_activate(bfwin, get_bmark_at_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter), FALSE);
@@ -1063,6 +1075,7 @@ bmark_event_mouseclick(GtkWidget * widget, GdkEventButton * event, Tbfwin * bfwi
 {
 	GtkTreePath *path;
 	gboolean show_bmark_specific = FALSE, show_file_specific = FALSE;
+	DEBUG_MSG("bmark_event_mouseclick, started\n");
 	if (gtk_tree_view_get_path_at_pos
 		(GTK_TREE_VIEW(bfwin->bmark), event->x, event->y, &path, NULL, NULL, NULL)) {
 		if (path) {
@@ -1071,12 +1084,12 @@ bmark_event_mouseclick(GtkWidget * widget, GdkEventButton * event, Tbfwin * bfwi
 			if (depth == 2) {
 				show_bmark_specific = TRUE;
 				show_file_specific = TRUE;
-				if (event->button == 1) {
+/*				if (event->button == 1) {
 					GtkTreeIter iter;
 					gtk_tree_model_get_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter, path);
 					bmark_activate(bfwin, get_bmark_at_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter),
 								   FALSE);
-				}
+				}*/
 			} else if (depth == 1) {
 				show_file_specific = TRUE;
 			}
@@ -1267,6 +1280,23 @@ bmark_search_changed(GtkEditable * editable, gpointer user_data)
 	gtk_tree_model_filter_refilter(bfwin->bmarkfilter);
 }
 
+static void
+bmark_cursor_changed_cb(GtkTreeView * treeview, gpointer user_data)
+{
+	Tbfwin *bfwin = user_data;
+	DEBUG_MSG("bmark_cursor_changed_cb, started\n");
+	GtkTreePath *path;
+	gtk_tree_view_get_cursor(treeview, &path, NULL);
+	if (path) {
+		if (gtk_tree_path_get_depth(path) == 2) {
+			GtkTreeIter iter;
+			gtk_tree_model_get_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter, path);
+			bmark_activate(bfwin, get_bmark_at_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter), FALSE);
+		}
+		gtk_tree_path_free(path);
+	}
+}
+
 /* Initialize bookmarks gui for window */
 GtkWidget *
 bmark_gui(Tbfwin * bfwin)
@@ -1331,6 +1361,7 @@ bmark_gui(Tbfwin * bfwin)
 	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
 	g_signal_connect(G_OBJECT(bfwin->bmark), "button-press-event", G_CALLBACK(bmark_event_mouseclick), bfwin);
 	g_signal_connect(G_OBJECT(bfwin->bmark), "row-activated", G_CALLBACK(bmark_row_activated), bfwin);
+	g_signal_connect(G_OBJECT(bfwin->bmark), "cursor-changed", G_CALLBACK(bmark_cursor_changed_cb), bfwin);
 	gtk_tree_view_expand_all(bfwin->bmark);
 	/*{
 	   GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(bfwin->bmark));
@@ -1785,6 +1816,9 @@ bmark_margin_get_next_bookmark(Tdocument * doc, gpointer * bmark)
 	gboolean cont;
 	GtkTextIter textit;
 	GtkTreeIter treeit = ((Tbmark *) * bmark)->iter;
+#ifndef SUPPRESS_DRAWING_CALLS
+	DEBUG_MSG("bmark_margin_get_next_bookmark\n");
+#endif
 	cont =
 		gtk_tree_model_iter_next(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
 								 &treeit);
@@ -1796,18 +1830,45 @@ bmark_margin_get_next_bookmark(Tdocument * doc, gpointer * bmark)
 	gtk_text_buffer_get_iter_at_mark(doc->buffer, &textit, ((Tbmark *) * bmark)->mark);
 	return gtk_text_iter_get_line(&textit);
 }
+/*
+this is called by the editor widget to show bookmarks in the left margin.
+it is called every time the margin is drawn to find the first line that has a bookmark 
+from there on the editor widget will call bmark_margin_get_next_bookmark() to paint 
+the next bookmark in the margin
 
-/* this is called by the editor widget to show bookmarks in the left margin.
- returns a line number for the Tbmark that bmark points to, or -1 if there is no bmark */
+returns a line number for the first Tbmark that this document has, or -1 if there is no bmark 
+*/
+
+gint 
+bmark_margin_get_first_bookmark(Tdocument *doc, gpointer *bmark) {
+	gboolean cont;
+	GtkTextIter textit;
+	GtkTreeIter treeit;
+	if (!doc->bmark_parent) return -1;
+#ifndef SUPPRESS_DRAWING_CALLS
+	DEBUG_MSG("bmark_margin_get_first_bookmark\n");
+#endif
+	cont = gtk_tree_model_iter_children(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
+		&treeit, doc->bmark_parent);
+	if (!cont) {
+		return -1;
+	}
+	gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &treeit,
+					   PTR_COLUMN, bmark, -1);
+	gtk_text_buffer_get_iter_at_mark(doc->buffer, &textit, ((Tbmark *) * bmark)->mark);
+	return gtk_text_iter_get_line(&textit);
+}
+
+/* bmark_margin_get_initial_bookmark
+this function is only used for the previous and the next function */
 gint
-bmark_margin_get_initial_bookmark(Tdocument * doc, GtkTextIter * fromit, gpointer * bmark)
+bmark_margin_get_initial_bookmark(Tdocument * doc, guint offset, gpointer * bmark)
 {
-	guint offset;
 	GtkTextIter textit;
 	if (!doc->bmark_parent) {
 		return -1;
 	}
-	offset = gtk_text_iter_get_offset(fromit);
+	DEBUG_MSG("bmark_margin_get_initial_bookmark, started\n");
 	*bmark = bmark_find_bookmark_before_offset(BFWIN(doc->bfwin), offset, doc->bmark_parent);	/* returns NULL if there is no existing bookmark *before* offset */
 	if (!*bmark) {
 		GtkTreeIter treeit;
@@ -1850,7 +1911,7 @@ bmark_add_backend(Tdocument * doc, GtkTextIter * itoffset, gint offset, const gc
 	}
 
 	m->mark = gtk_text_buffer_create_mark(doc->buffer, NULL, &it, TRUE);
-	DEBUG_MSG("bmark_add_backend, mark=%p, create GtkTextMark %p\n", m, m->mark);
+	DEBUG_MSG("bmark_add_backend, Tbmark=%p, create GtkTextMark %p\n", m, m->mark);
 	m->uri = bmark_uri_from_doc(doc);
 	m->is_temp = is_temp;
 	m->text = g_strdup(text);
@@ -1938,75 +1999,85 @@ can we make this function faster? when adding bookmarks from a search this funct
 a lot of time, perhaps that can be improved
 */
 static Tbmark *
-bmark_get_bmark_at_iter(Tdocument * doc, GtkTextIter * iter, gint offset)
+bmark_get_bmark_at_offset(Tdocument * doc, gint offset)
 {
 	GtkTextIter sit, eit;
 	GtkTreeIter tmpiter;
 	gint linenum;
-	sit = *iter;
+
+	if (!DOCUMENT(doc)->bmark_parent) {	
+		return NULL;
+	}
+	gtk_text_buffer_get_iter_at_offset(doc->buffer, &sit, offset);
 	linenum = gtk_text_iter_get_line(&sit);
 	eit = sit;
 	gtk_text_iter_set_line_offset(&sit, 0);
 	gtk_text_iter_forward_to_line_end(&eit);
+	DEBUG_MSG("bmark_get_bmark_at_offset, offset=%d\n",offset);
 	/* check for existing bookmark in this place */
-	if (DOCUMENT(doc)->bmark_parent) {
-		GtkTextIter testit;
-		Tbmark *m, *m2;
-		/* the next function is probably the slowest since it jumps through the listmodel 
-		   to find the right bookmark */
-		m = bmark_find_bookmark_before_offset(BFWIN(doc->bfwin), offset, doc->bmark_parent);
-		if (m == NULL) {
-			DEBUG_MSG("bmark_get_bmark_at_iter, m=NULL, get first child\n");
-			if (gtk_tree_model_iter_children
-				(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &tmpiter,
-				 doc->bmark_parent)) {
-				gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
-								   &tmpiter, PTR_COLUMN, &m2, -1);
-				gtk_text_buffer_get_iter_at_mark(doc->buffer, &testit, m2->mark);
-				if (gtk_text_iter_get_line(&testit) == linenum) {
-					return m2;
-				}
-			}
-		} else {
-			gtk_text_buffer_get_iter_at_mark(doc->buffer, &testit, m->mark);
-			DEBUG_MSG("bmark_get_bmark_at_iter, m=%p, has linenum=%d\n", m, gtk_text_iter_get_line(&testit));
+	GtkTextIter testit;
+	Tbmark *m, *m2;
+	/* the next function is probably the slowest since it jumps through the listmodel 
+	   to find the right bookmark */
+	m = bmark_find_bookmark_before_offset(BFWIN(doc->bfwin), offset, doc->bmark_parent);
+	if (m == NULL) {
+		DEBUG_MSG("bmark_get_bmark_at_offset, m=NULL, get first child\n");
+		if (gtk_tree_model_iter_children
+			(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &tmpiter,
+			 doc->bmark_parent)) {
+			gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
+							   &tmpiter, PTR_COLUMN, &m2, -1);
+			gtk_text_buffer_get_iter_at_mark(doc->buffer, &testit, m2->mark);
 			if (gtk_text_iter_get_line(&testit) == linenum) {
-				return m;
+				return m2;
 			}
-			tmpiter = m->iter;
-			if (gtk_tree_model_iter_next
-				(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &tmpiter)) {
-				gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
-								   &tmpiter, PTR_COLUMN, &m2, -1);
-				gtk_text_buffer_get_iter_at_mark(doc->buffer, &testit, m2->mark);
-				if (gtk_text_iter_get_line(&testit) == linenum) {
-					return m2;
-				}
+		}
+	} else {
+		gtk_text_buffer_get_iter_at_mark(doc->buffer, &testit, m->mark);
+		DEBUG_MSG("bmark_get_bmark_at_offset, m=%p, has linenum=%d\n", m, gtk_text_iter_get_line(&testit));
+		if (gtk_text_iter_get_line(&testit) == linenum) {
+			return m;
+		}
+		tmpiter = m->iter;
+		if (gtk_tree_model_iter_next
+			(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &tmpiter)) {
+			gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
+							   &tmpiter, PTR_COLUMN, &m2, -1);
+			gtk_text_buffer_get_iter_at_mark(doc->buffer, &testit, m2->mark);
+			if (gtk_text_iter_get_line(&testit) == linenum) {
+				return m2;
 			}
 		}
-		DEBUG_MSG("bmark_get_bmark_at_iter, nothing found at this line, return NULL\n");
-		return NULL;
-
 	}
-	DEBUG_MSG("bmark_get_bmark_at_iter, no existing bookmark found, return NULL\n");
+	DEBUG_MSG("bmark_get_bmark_at_offset, nothing found at this line, return NULL\n");
 	return NULL;
 }
 
 static Tbmark *
-bmark_get_bmark_at_line(Tdocument * doc, gint line)
+bmark_get_bmark_at_line(Tdocument * doc, guint line)
 {
-	GtkTextIter iter;
-	gtk_text_buffer_get_iter_at_line(doc->buffer, &iter, line);
-	return bmark_get_bmark_at_iter(doc, &iter, gtk_text_iter_get_offset(&iter));
+	gboolean cont;
+	GtkTreeIter treeit;
+	GtkTextIter textit;
+	DEBUG_MSG("bmark_get_bmark_at_line, started\n");
+	if (!doc->bmark_parent) return NULL;
+	cont = gtk_tree_model_iter_children(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &treeit,doc->bmark_parent);
+	while (cont) {
+		guint bline;
+		Tbmark *bmark;
+		gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &treeit, PTR_COLUMN, &bmark,-1);
+		gtk_text_buffer_get_iter_at_mark(doc->buffer, &textit, bmark->mark);
+		bline = gtk_text_iter_get_line(&textit);
+		if (bline == line) {
+			return bmark;
+		} else if (bline > line) {
+			return NULL;
+		}
+		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &treeit);
+	}
+	return NULL;
 }
 
-static Tbmark *
-bmark_get_bmark_at_offset(Tdocument * doc, gint offset)
-{
-	GtkTextIter iter;
-	gtk_text_buffer_get_iter_at_offset(doc->buffer, &iter, offset);
-	return bmark_get_bmark_at_iter(doc, &iter, offset);
-}
 
 /**
  * bmark_add_extern
@@ -2025,14 +2096,12 @@ bmark_add_extern(Tdocument * doc, gint offset, const gchar * name, const gchar *
 	if (!doc)
 		return;
 	DEBUG_MSG("adding bookmark at offset %d with name %s\n", offset, name);	/* dummy */
-	if (!bmark_get_bmark_at_offset(doc, offset)) {
-		if (text) {
-			bmark_add_backend(doc, NULL, offset, (name) ? name : "", text, is_temp);
-		} else {
-			gchar *tmp = bmark_text_for_offset(doc, NULL, offset);
-			bmark_add_backend(doc, NULL, offset, (name) ? name : "", tmp, is_temp);
-			g_free(tmp);
-		}
+	if (text) {
+		bmark_add_backend(doc, NULL, offset, (name) ? name : "", text, is_temp);
+	} else {
+		gchar *tmp = bmark_text_for_offset(doc, NULL, offset);
+		bmark_add_backend(doc, NULL, offset, (name) ? name : "", tmp, is_temp);
+		g_free(tmp);
 	}
 }
 
@@ -2090,6 +2159,7 @@ gchar *
 bmark_get_tooltip_for_line(Tdocument * doc, gint line)
 {
 	Tbmark *bmark;
+	DEBUG_MSG("bmark_get_tooltip_for_line %d\n",line);
 	bmark = bmark_get_bmark_at_line(doc, line);
 	if (!bmark || !bmark->text)
 		return NULL;
@@ -2100,21 +2170,6 @@ bmark_get_tooltip_for_line(Tdocument * doc, gint line)
 	return g_markup_escape_text(bmark->text, -1);
 }
 
-
-/*gchar *
-bmark_get_tooltip_for_line(Tdocument *doc, gint line)
-{
-	Tbmark *bmark;
-	bmark = bmark_get_bmark_at_line(doc, line);
-	if (!bmark || !bmark->text)
-		return NULL;
-	if (bmark->text && bmark->name) 
-		return g_strconcat(bmark->name," ", bmark->text, NULL);
-	if (bmark->name)
-		return g_strdup(bmark->name);
-	return g_strdup(bmark->text);
-}
-*/
 void
 bmark_del_at_bevent(Tdocument * doc)
 {
