@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * compatibility.c
  *
- * Copyright (C) 2014 Olivier Sessink
+ * Copyright (C) 2014-2023 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,3 +43,32 @@ char *strcasestr(char *a, char *b)
 	return(NULL);
 }
 #endif
+
+#if !GLIB_CHECK_VERSION(2,40,0)
+gboolean
+g_str_is_ascii(const gchar *str)
+{
+	gsize i;
+	for (i = 0; str[i]; i++)
+	if (str[i] & 0x80)
+		return FALSE;
+	return TRUE;
+}
+
+gchar *
+g_str_to_ascii_minimal(const gchar *pattern)
+{
+	gsize i=0, j=0;
+	gchar *retval = g_malloc(strlen(pattern));
+	while(pattern[i]) {
+		if (pattern[i] & 0x80)
+			retval[j]=pattern[i];
+			j++;
+		i++;
+	}
+	retval[i] = '\0';
+	return retval;
+}
+
+#endif
+ 
\ No newline at end of file
