@@ -240,11 +240,11 @@ dialog_color_button_in_table(const gchar * color, const gchar * title, GtkWidget
 		button = gtk_color_button_new_with_color(&gdkcolor);
 	} else {
 		button = gtk_color_button_new();
-		gtk_color_button_set_alpha(GTK_COLOR_BUTTON(button), 0);
+		/*gtk_color_button_set_alpha(GTK_COLOR_BUTTON(button), 0);*/
 	}
 
 	gtk_color_button_set_title(GTK_COLOR_BUTTON(button), title);
-	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(button), TRUE);
+	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(button), FALSE);
 	gtk_table_attach(GTK_TABLE(table), button, left_attach, right_attach, top_attach, bottom_attach, GTK_FILL,
 					 GTK_SHRINK, 0, 0);
 
@@ -517,14 +517,19 @@ dialog_label_new(const gchar * labeltext, gfloat xalign, gfloat yalign, GtkWidge
 GtkWidget *
 dialog_vbox_new(GtkWidget * box)
 {
-	GtkWidget *alignment, *vbox;
-
-	alignment = gtk_alignment_new(0, 0, 1, 1);
+	GtkWidget *vbox;
+	vbox = gtk_vbox_new(FALSE, 8);
+#if GTK_CHECK_VERSION(3,14,0)
+	gtk_widget_set_halign(vbox,GTK_ALIGN_FILL);
+	gtk_widget_set_valign(vbox,GTK_ALIGN_FILL);
+	gtk_box_pack_start(GTK_BOX(box), vbox, FALSE, FALSE, 8);
+#else
+	GtkWidget *alignment;
+	alignment = gtk_alignment_new(0/* left */, 0/*top*/, 1/*x expand */, 1/* y expand */);
 	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 8, 4);
 	gtk_box_pack_start(GTK_BOX(box), alignment, FALSE, FALSE, 0);
-	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(alignment), vbox);
-
+#endif
 	return vbox;
 }
 
