@@ -438,6 +438,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.restore_dimensions, "restore_dimensions:", 1, TRUE);
 
 	init_prop_integer(&config_rc, &main_v->props.left_panel_left, "left_panel_left:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.restore_search_scope, "restore_search_scope:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.hide_bars_on_fullscreen, "hide_bars_on_fullscreen:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.cursor_size, "cursor_size:", 4, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.highlight_cursor, "highlight_cursor:", 0, TRUE);
@@ -1276,8 +1277,10 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 
 	init_prop_integer(&configlist, &session->snr3_type, "snr_type:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_replacetype, "snr_replacetype:", 1, FALSE);
-	/*	no longer stored, see https://sourceforge.net/p/bluefish/tickets/26/ 
-	init_prop_integer(&configlist, &session->snr3_scope, "snr_scope:", 1, FALSE);*/
+	/*	no longer stored by default, see https://sourceforge.net/p/bluefish/tickets/26/ */
+	if (main_v->props.restore_search_scope) {
+		init_prop_integer(&configlist, &session->snr3_scope, "snr_scope:", 1, FALSE);
+	}
 	init_prop_integer(&configlist, &session->snr3_casesens, "snr_casesens:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_showinoutputbox, "snr_showinoutputbox:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_ignorebackupfiles, "snr_ignorebackupfiles:", 1, FALSE);
